#ifndef LNM_HTTP_REQ
#define LNM_HTTP_REQ

#include <regex.h>
#include <stdlib.h>

#include "picohttpparser.h"

#include "lnm/http/consts.h"

#define LNM_HTTP_MAX_REQ_HEADERS 32
#define LNM_HTTP_MAX_REGEX_GROUPS 4

/**
 * Represents the parsed HTTP request
 */
typedef struct lnm_http_req {
  size_t len;
  int minor_version;
  lnm_http_method method;
  struct {
    const char *s;
    size_t len;
    regmatch_t groups[LNM_HTTP_MAX_REGEX_GROUPS];
  } path;
  struct {
    const char *s;
    size_t len;
  } query;
  struct {
    struct phr_header arr[LNM_HTTP_MAX_REQ_HEADERS];
    size_t len;
  } headers;
} lnm_http_req;

typedef enum lnm_http_parse_err {
  lnm_http_parse_err_ok = 0,
  lnm_http_parse_err_incomplete,
  lnm_http_parse_err_invalid,
  lnm_http_parse_err_unknown_method,
} lnm_http_parse_err;

/**
 * Try to parse the given buffer into an HTTP request.
 *
 * @param req request to store parsed data in
 * @param buf buffer to parse; might be modified
 * @param len length of buf
 */
lnm_http_parse_err lnm_http_req_parse(lnm_http_req *req, char *buf, size_t len);

#endif
